<?php
namespace Service;
use Util\Utilities;
use Util\Instance\RepositoryInstance;
use Util\Instance\FactoryInstance;
use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment as TeknisaEnvironment;
use Exception\CustomException;



class Repositorio {

    protected $entityManager;

    public function __construct(
        EntityManager $entityManager, TeknisaEnvironment $environment
    ){
        $this->entityManager = $entityManager;
        $this->environment = $environment;
    }

    public function findRepositorio($filters, $page, $pageSize)
    {
        $conditions = Utilities::createQueryFiltered($filters);
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $currentUser = $this->environment->getCurrentUserId();
        return RepositoryInstance::getRepositorio()->getAllRepositories($conditions, $currentUser, $limit, $offset);
    }

    /**
     * @param $repositorioId
     * @param $idativo
     * @param $nmrepositorio
     * @param $dsurlrepositorio
     * @param $diretorio
     * @param $repositorioOperId
     * @throws CustomException
     * @throws \Doctrine\DBAL\ConnectionException
     */
    public function saveRepositorio($repositorioId, $idativo, $nmrepositorio, $dsurlrepositorio, $diretorio, $repositorioOperId)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            $nrorg = $this->environment->getCurrentOrganizationId();
            $currentUser = $this->environment->getCurrentUserId();
            $repositorioRepository = RepositoryInstance::getMtRepositorio();
            $repositorioEntity = $repositorioRepository->findOneBy(array('id' => $repositorioId));
            $entidaderepositorio = FactoryInstance::getRepositorio()->saveRepositorio($repositorioEntity ,$nrorg, $currentUser, $idativo, $nmrepositorio, $dsurlrepositorio);
            $this->entityManager->persist($entidaderepositorio);
            $this->entityManager->flush();

            if($diretorio || $repositorioOperId){
                $repositorioOperRepository = RepositoryInstance::getMtRepositorioOperador();
                $repositorioOperEntity = $repositorioOperRepository->findOneBy(array('id' => $repositorioOperId));
                if(!empty($diretorio)) {
                    $entidadediretorio = FactoryInstance::getRepositorio()->saveDiretorio($repositorioOperEntity, $entidaderepositorio->getId(), $diretorio, $nrorg, $currentUser, $idativo);
                    $this->entityManager->persist($entidadediretorio);
                } else {
                    $this->entityManager->remove($repositorioOperEntity);
                }
                $this->entityManager->flush();
            };

            $connection->commit();
        } catch (\Exception $ex) {
            $connection->rollback();
            if(empty($nmrepositorio)) {
                throw new CustomException('ERR_NAME_NOT_DEFINED');
            } else if(empty($dsurlrepositorio)){
                throw new CustomException('ERR_URL_NOT_DEFINED');
            }
            throw $ex;
        }

    }

    /**
     * @param $repositorioId
     * @param $repositorioOperId
     * @throws CustomException
     * @throws \Doctrine\DBAL\ConnectionException
     */
    public function removeRepositorio($repositorioId, $repositorioOperId)
    {
        $repositorioRepository = RepositoryInstance::getMtRepositorio();
        $currentUser = $this->environment->getCurrentUserId();
        $validateRemoval = RepositoryInstance::getRepositorio()->validateRepositorioDelete($repositorioId, $currentUser);
        $validateStrucureDesign = RepositoryInstance::getRepositorio()->validateStructureDesign($repositorioId);
        if($validateStrucureDesign){
            throw new CustomException('ERR_REMOVE_DESIGN_STRUCTURE');
        }
        if(!$validateRemoval) {
            $connection = $this->entityManager->getConnection();
            $repositorioEntity = $repositorioRepository->findOneBy(array('id' => $repositorioId));
            try {
                $connection->beginTransaction();
                if($repositorioOperId){
                    $repositorioOperRepository = RepositoryInstance::getMtRepositorioOperador();
                    $repositorioOperEntity = $repositorioOperRepository->findOneBy(array('id' => $repositorioOperId));
                    $this->entityManager->remove($repositorioOperEntity);
                    $this->entityManager->flush();
                }
                $this->entityManager->remove($repositorioEntity);
                $this->entityManager->flush();
                $connection->commit();
            } catch (\Exception $ex) {
                $connection->rollBack();
                throw $ex;
            }
        } else {
            throw new CustomException('ERR_REMOVE_REPOSITORIO');
        }
    }
}