<?php
namespace Service;
use \Doctrine\ORM\EntityManager;
use Util\Utilities;
use Util\Instance\GeneralProvider;
use Util\Messages;
use Util\Instance\RepositoryInstance;
use Util\Instance\FactoryInstance;
use Teknisa\Libs\Util\Environment as TeknisaEnvironment;




class Table {

    public function __construct(
         TeknisaEnvironment $environment
    ){
        $this->environment = $environment;
    }
    public function findTable($filters, $page, $pageSize)
    {
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $conditions = Utilities::createQueryFiltered($filters);
        return RepositoryInstance::getTable()->getAllTables($conditions, $limit, $offset);
    }

    public function findTableObservations($table)
    {
        return RepositoryInstance::getTable()->getTableObservatrion($table)[0]["DSTABELABD"];
    }

    public function findtableColumnObservations($column, $table)
    {
        $coluna = RepositoryInstance::getTable()->gettableColumnObservations($column, $table);
        if($coluna){
            return $coluna[0]["DSCOLUNA"];
        }else{
            return null;
        }
    }

    public function findTableColumns($filters, $primaryKey, $page, $pageSize)
    {
        if($primaryKey){
            if($primaryKey == 'Y'){
                $pkFilter = " AND PK IS NOT NULL";
            }else {
                $pkFilter = " AND PK IS NULL";
            }
        }else {
            $pkFilter = "";
        }
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $filterString = Utilities::createQueryFiltered($filters);
        return RepositoryInstance::getTable()->getAllTableColumns($filterString, $pkFilter, $limit, $offset);
    }

    public function findTableConstraints($filters, $page, $pageSize)
    {
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $filterString = Utilities::createQueryFiltered($filters);
        return RepositoryInstance::getTable()->getAllTableConstraints($filterString, $limit, $offset);
    }

    public function findTableTrigger($filters, $page, $pageSize)
    {
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $filterString = Utilities::createQueryFiltered($filters);
        return RepositoryInstance::getTable()->getAllTableTriggers($filterString, $limit, $offset);
    }

    public function findTableConstraintColumns($filters, $page, $pageSize, $constraintIsFk)
    {
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $filterString = Utilities::createQueryFiltered($filters);
        if($constraintIsFk == 'S'){
            return RepositoryInstance::getTable()->getAllTableFkConstraintColumns($filterString, $limit, $offset);

        }else{
            return RepositoryInstance::getTable()->getAllTableConstraintColumns($filterString, $limit, $offset);
        }
    }

    public function saveTableObservations($table, $observation)
    {
        $repository = RepositoryInstance::getTabela();
        $entityManager = GeneralProvider::getEntityManager();
        $connection    = $entityManager->getConnection();
        try {
            $connection->beginTransaction();

            $tabelaEntity = $repository->findOneBy(array('nmtabelabd' => $table));
            $currentUser = $this->environment->getCurrentUserId();
            $nrorg = $this->environment->getCurrentOrganizationId();
            $tableFactory = FactoryInstance::getTable()->saveObservation($tabelaEntity, $table, $observation, $currentUser, $nrorg);
            $entityManager->persist($tableFactory);
            $entityManager->flush();
            $connection->commit();
            return $tabelaEntity->getNmtabelabd();
        } catch (\Exception $ex) {
            $connection->rollback();
            throw $ex;
        }


    }

    public function saveTableColumnObservation($tabela, $coluna, $observation)
    {
        $repository = RepositoryInstance::getColuna();
        $entityManager = GeneralProvider::getEntityManager();
        $connection    = $entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $colunaEntity = $repository->findOneBy(array('nmtabelabd' => $tabela, 'nmcoluna' => $coluna));
            $currentUser = $this->environment->getCurrentUserId();
            $nrorg = $this->environment->getCurrentOrganizationId();
            $tableFactory = FactoryInstance::getTable()->saveColumnObservation($colunaEntity, $tabela, $coluna, $observation, $currentUser, $nrorg);
            $entityManager->persist($tableFactory);
            $entityManager->flush();
            $connection->commit();
            return $colunaEntity->getNmcoluna();
        } catch (\Exception $ex) {
            $connection->rollback();
            throw $ex;
        }


    }
}

